# 1 "wikicreole.mll"
 
(* Ocsimore
 * Copyright (C) 2008
 * Laboratoire PPS - Universit Paris Diderot - CNRS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)
(**
   Parser for Wikicreole
   @author Jrme Vouillon
   @author Vincent Balat
*)

exception Eof
exception Unrecognized_char

type attribs = (string * string) list

type (+'a, +'b, +'c, +'d) ext_kind =
  [ `Flow5 of 'a
  | `Flow5_link of 'b
  | `Phrasing_without_interactive of 'c
  | `Phrasing_link of 'd ]

type style =
    Bold | Italic | Underlined | Linethrough |
        Monospace | Superscripted | Subscripted

type list_kind = Unordered | Ordered

type (-'param, +'res) plugin =
  'param ->
    attribs -> (** Xml-like attributes for the extension (eg val='foo') *)
      string option -> (** content for the extension, after the '|' *)
        'res

module type RawBuilder = sig

  type href
  type param
  type phrasing_without_interactive
  type phrasing
  type flow
  type flow_without_interactive
  type uo_list

  val chars : string -> phrasing_without_interactive
  val strong_elem : attribs -> phrasing list -> phrasing_without_interactive
  val em_elem : attribs -> phrasing list -> phrasing_without_interactive
  val br_elem : attribs -> phrasing_without_interactive
  val img_elem : attribs -> href -> string -> phrasing_without_interactive
  val tt_elem : attribs -> phrasing list -> phrasing_without_interactive
  val monospace_elem : attribs -> phrasing list -> phrasing_without_interactive
  val underlined_elem : attribs -> phrasing list -> phrasing_without_interactive
  val linethrough_elem : attribs -> phrasing list -> phrasing_without_interactive
  val subscripted_elem : attribs -> phrasing list -> phrasing_without_interactive
  val superscripted_elem : attribs -> phrasing list -> phrasing_without_interactive
  val nbsp : phrasing_without_interactive
  val endash : phrasing_without_interactive
  val emdash : phrasing_without_interactive
  val a_elem_phrasing : attribs -> href -> phrasing_without_interactive list -> phrasing
  val a_elem_flow : attribs -> href -> flow_without_interactive list -> flow
  val make_href : param -> string -> string option -> href
  (** the string option is the fragment part of the URL (#...)*)
  val p_elem : attribs -> phrasing list -> flow_without_interactive
  val pre_elem : attribs -> string list -> flow_without_interactive
  val h1_elem : attribs -> phrasing list -> flow_without_interactive
  val h2_elem : attribs -> phrasing list -> flow_without_interactive
  val h3_elem : attribs -> phrasing list -> flow_without_interactive
  val h4_elem : attribs -> phrasing list -> flow_without_interactive
  val h5_elem : attribs -> phrasing list -> flow_without_interactive
  val h6_elem : attribs -> phrasing list -> flow_without_interactive
  val section_elem : attribs -> flow list -> flow_without_interactive

  val ul_elem : attribs -> (phrasing list * uo_list option * attribs) list -> uo_list
  val ol_elem : attribs -> (phrasing list * uo_list option * attribs) list -> uo_list

  val dl_elem : attribs -> (bool * phrasing list * attribs) list -> flow_without_interactive
  val hr_elem : attribs -> flow_without_interactive

  val table_elem : attribs ->
      ((bool * attribs * phrasing list) list * attribs) list -> flow_without_interactive

  val phrasing : phrasing_without_interactive -> phrasing
  val flow : flow_without_interactive -> flow
  val list : uo_list -> flow_without_interactive

  val error : string -> phrasing_without_interactive

end

module type Builder = sig

  include RawBuilder

  type syntax_extension =
      (flow,
       (href * attribs * flow_without_interactive),
       phrasing_without_interactive,
       (href * attribs * phrasing_without_interactive)) ext_kind

  val plugin : string -> bool * (param, syntax_extension) plugin
  val plugin_action :  string -> int -> int -> (param, unit) plugin
  val link_action : string -> string option -> attribs -> int * int -> param -> unit

end

module Make(B : Builder) = struct

  type stack =
    | Style of style * B.phrasing list * attribs * stack
    | Link of B.href * attribs * stack
  (* Not that we do not save anything in the case of links, as
     links cannot be nested *)
    | Paragraph of attribs
    | Heading of int * attribs
    | List_item of attribs * stack
    | List of
	list_kind * (B.phrasing list * B.uo_list option * attribs) list
      * attribs * stack
    | Descr_def of attribs * stack
    | Descr_title of attribs * stack
    | Descr of attribs * stack
    | Table of ((bool * attribs * B.phrasing list) list * attribs) list * attribs
    | Row of (bool * attribs * B.phrasing list) list * attribs * stack
    | Entry of bool * attribs * stack

  type ctx =
      { param : B.param;
	mutable italic : bool;
	mutable bold : bool;
	mutable monospace : bool;
	mutable underlined : bool;
	mutable linethrough : bool;
	mutable subscripted : bool;
	mutable superscripted : bool;
	mutable heading : bool;
	mutable link : bool;
	mutable list_level : int;
	mutable phrasing_mix : B.phrasing list;
	mutable link_content : B.phrasing_without_interactive list;
	mutable pre_content : string list;
	mutable list : (B.phrasing list * B.uo_list option * attribs) list;
	mutable descr : (bool * B.phrasing list * attribs) list;
	mutable flow : (int * B.flow list) list;
	mutable stack : stack;
	mutable sectioning: bool;
      }

  let count c s =
    let n = ref 0 in
    for i = 0 to String.length s - 1 do if s.[i] = c then incr n done;
    !n

  let push c v =
    match c.stack with
      | Link _ -> c.link_content <- v :: c.link_content
      | _      -> c.phrasing_mix <- B.phrasing v :: c.phrasing_mix

let push_string c s = push c (B.chars s)

let push_chars c lexbuf = push_string c (Lexing.lexeme lexbuf)

let push_flow c e = match c.flow with
  | (lvl, flow) :: flows -> c.flow <- (lvl, e :: flow) :: flows
  | [] -> assert false

let rec end_section c lvl =
  match c.flow with
  | (lvl', flow) :: flows when lvl <= lvl' ->
      c.flow <- flows;
      push_flow c (B.flow (B.section_elem [(* TODO*)] (List.rev flow)));
      end_section c lvl
  | _ -> ()

let get_section_lvl c = match c.flow with
  | (lvl, _) :: _ -> lvl
  | [] -> assert false

let push_section c lvl =
  assert (get_section_lvl c < lvl);
  if c.sectioning then c.flow <- (lvl,[]) :: c.flow

let read_attribs att parse_attribs c lexbuf =
  try
    if att = "@@"
    then match parse_attribs 1 [] [] c lexbuf with
      | [] -> []
      | a::_ -> a
    else []
  with Eof -> [] (*VVV ??? *)

let read_list_attribs att parse_attribs c lexbuf =
  try
    if att = "@@"
    then match parse_attribs 2 [] [] c lexbuf with
      | [] -> ([], [])
      | a::b::_ -> (b, a)
      | [a] -> ([], a)
    else ([], [])
  with Eof -> ([], []) (*VVV ??? *)

let read_table_attribs att parse_attribs c lexbuf =
  try
    if att = "@@"
    then match parse_attribs 3 [] [] c lexbuf with
      | [] -> ([], [], [])
      | a::b::c::_ -> (c, b, a)
      | [a; b] -> ([], b, a)
      | [a] -> ([], [], a)
    else ([], [], [])
  with Eof -> ([], [], []) (*VVV ??? *)

let get_style c style =
  match style with
    | Bold -> c.bold
    | Italic -> c.italic
    | Monospace -> c.monospace
    | Underlined -> c.underlined
    | Linethrough -> c.linethrough
    | Subscripted -> c.subscripted
    | Superscripted -> c.superscripted

let set_style c style v =
  match style with
    | Bold -> c.bold <- v
    | Italic -> c.italic <- v
    | Monospace -> c.monospace <- v
    | Underlined -> c.underlined <- v
    | Linethrough -> c.linethrough <- v
    | Subscripted -> c.subscripted <- v
    | Superscripted -> c.superscripted <- v

let pop_style c style phrasing attribs stack =
  let elt =
    match style with
      | Bold   -> B.strong_elem attribs
      | Italic -> B.em_elem attribs
      | Monospace -> B.monospace_elem attribs
      | Underlined -> B.underlined_elem attribs
      | Linethrough -> B.linethrough_elem attribs
      | Subscripted -> B.subscripted_elem attribs
      | Superscripted -> B.superscripted_elem attribs
  in
  let phrasing' = c.phrasing_mix in
  c.stack <- stack;
  c.phrasing_mix <- phrasing;
  push c (elt (List.rev phrasing'));
  set_style c style false

let style_change c style att parse_attribs lexbuf =
  let atts = read_attribs att parse_attribs c lexbuf in
  if get_style c style then begin
    match c.stack with
      Style (s, phrasing, attribs, stack) when s = style ->
        pop_style c style phrasing attribs stack;
    | _ ->
        push_string c "**";
        push_string c att
  end else begin
    c.stack <- Style (style, c.phrasing_mix, atts, c.stack);
    c.phrasing_mix <- [];
    set_style c style true
  end

let pop_link c addr attribs stack =
  c.stack <- stack;
  c.phrasing_mix <-
    B.a_elem_phrasing attribs addr (List.rev c.link_content) :: c.phrasing_mix;
  c.link_content <- [];
  c.link <- false

let close_entry c =
  match c.stack with
    Entry (heading, attribs, Row (entries, row_attribs, stack)) ->
      c.stack <- Row ((heading, attribs, List.rev c.phrasing_mix) :: entries,
                      row_attribs,
                      stack);
      c.phrasing_mix <- [];
      true
  | Row _ ->
      true
  | Table _ ->
      (*VVV attribs? *)
      c.stack <- Row ([(false, [], List.rev c.phrasing_mix)], [], c.stack);
      c.phrasing_mix <- [];
      true
  | _ ->
      false

let close_row c =
  close_entry c &&
  match c.stack with
    Row (entries, row_attribs, Table (rows, table_attribs)) ->
      c.stack <- Table (((List.rev entries, row_attribs) :: rows),
                        table_attribs);
      true
  | Table _ ->
      true
  | _ ->
      assert false

let close_descr_entry c =
  match c.stack with
    | Descr_def (attribs, stack) ->
        c.stack <- stack;
        c.descr <- (false, List.rev c.phrasing_mix, attribs) :: c.descr;
        c.phrasing_mix <- [];
        true
    | Descr_title (attribs, stack) ->
        c.stack <- stack;
        c.descr <- (true, List.rev c.phrasing_mix, attribs) :: c.descr;
        c.phrasing_mix <- [];
        true
    | _ ->
        false


let rec end_paragraph c lev =
  match c.stack with
    Style (style, phrasing, attribs, stack) ->
      pop_style c style phrasing attribs stack;
      end_paragraph c lev
  | Link (addr, attribs, stack) ->
      pop_link c addr attribs stack;
      end_paragraph c lev
  | Paragraph attribs ->
      if c.phrasing_mix <> [] then begin
        push_flow c (B.flow (B.p_elem attribs (List.rev c.phrasing_mix)));
        c.phrasing_mix <- []
      end;
      c.stack <- Paragraph []
  | Heading (l, attribs) ->
      let f =
        match l with
          | 1 -> B.h1_elem
          | 2 -> B.h2_elem
          | 3 -> B.h3_elem
          | 4 -> B.h4_elem
          | 5 -> B.h5_elem
          | _ -> B.h6_elem
      in
      push_flow c (B.flow (f attribs (List.rev c.phrasing_mix)));
      c.phrasing_mix <- [];
      c.heading <- false;
      c.stack <- Paragraph []
  | List_item (attribs, stack) ->
      c.list <- (List.rev c.phrasing_mix, None, attribs) :: c.list;
      c.stack <- stack;
      c.phrasing_mix <- [];
      end_paragraph c lev
  | List (kind, lst, attribs, stack) ->
      if lev < c.list_level then begin
        c.list_level <- c.list_level - 1;
        let elt =
          match kind with
            Unordered -> B.ul_elem
          | Ordered   -> B.ol_elem
        in
        let cur_lst = elt attribs (List.rev c.list) in
        if c.list_level = 0 then
          push_flow c (B.flow (B.list cur_lst))
        else begin
          match lst with
            (l, None, attribs) :: rem ->
              c.list <- (l, Some cur_lst, attribs) :: rem;
          | _                -> assert false
        end;
        c.stack <- stack;
        end_paragraph c lev
      end
  | Descr_def (attribs, stack) ->
      c.descr <- (false, List.rev c.phrasing_mix, attribs) :: c.descr;
      c.stack <- stack;
      c.phrasing_mix <- [];
      end_paragraph c lev
  | Descr_title (attribs, stack) ->
      c.descr <- (true, List.rev c.phrasing_mix, attribs) :: c.descr;
      c.stack <- stack;
      c.phrasing_mix <- [];
      end_paragraph c lev
  | Descr (attribs, stack) ->
      push_flow c (B.flow (B.dl_elem attribs (List.rev c.descr)));
      c.stack <- stack;
      end_paragraph c lev
  | Entry _ ->
      ignore (close_row c);
      end_paragraph c lev
  | Row _ ->
      assert false
  | Table (rows, attribs) ->
      push_flow c (B.flow (B.table_elem attribs (List.rev rows)));
      c.stack <- Paragraph []

let rec correct_kind_rec stack kind n =
  match stack with
    List_item (_, stack) ->
      correct_kind_rec stack kind n
  | List (k, _lst, _, stack) ->
      if n = 0 then k = kind else
      correct_kind_rec stack kind (n - 1)
  | Style (_, _, _, stack) ->
      correct_kind_rec stack kind n
  | Link _ | Heading _ | Paragraph _ | Entry _ | Row _ | Table _
  | Descr _ | Descr_title _ | Descr_def _ ->
      assert false

let correct_kind c kind lev =
  lev = c.list_level + 1
    ||
  (lev <= c.list_level &&
   correct_kind_rec c.stack kind (c.list_level - lev))

let start_list_item c kind lev att parse_attribs lexbuf =
  let correct = correct_kind c kind lev in
  if lev = 1 || correct then begin
    (* If we have an item of a different kind at level 1, we close the
       previous list and start a new one of the right kind *)
    end_paragraph c (if correct then lev else 0);
    let (list_attribs, item_attribs) =
      read_list_attribs att parse_attribs c lexbuf
    in
    if lev = c.list_level then begin
      c.stack <- List_item (item_attribs, c.stack)
    end else (* if lev = c.list_level + 1 then *) begin
      c.list_level <- lev;
      c.stack <- List_item (item_attribs,
                            List (kind, c.list, list_attribs, c.stack));
      c.list <- []
    end;
    true
  end else
    false

let start_table_row c heading (table_attribs, row_attribs, entry_attribs) =
  if not (close_row c) then begin
    end_paragraph c 0;
    c.stack <- Table ([], table_attribs)
  end;
  c.stack <- Entry (heading,
                    entry_attribs,
                    Row ([], row_attribs, c.stack))


# 459 "wikicreole.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base =
   "\000\000\001\000\002\000\002\000\001\000\003\000\004\000\005\000\
    \006\000\010\000\040\000\001\000\255\255\009\000\015\000\016\000\
    \017\000\022\000\020\000\254\255\023\000\253\255\024\000\252\255\
    \025\000\251\255\026\000\250\255\002\000\005\000\028\000\249\255\
    \002\000\007\000\027\000\003\000\041\000\248\255\004\000\029\000\
    \005\000\030\000\031\000\247\255\032\000\246\255\245\255\087\000\
    \229\255\093\000\098\000\012\000\183\000\222\000\020\001\093\001\
    \132\001\186\001\240\001\055\002\109\002\163\002\202\002\000\003\
    \054\003\093\003\147\003\009\000\255\255\038\000\045\000\254\255\
    \046\000\048\000\049\000\253\255\050\000\248\255\051\000\052\000\
    \252\255\053\000\054\000\251\255\055\000\056\000\250\255\057\000\
    \063\000\249\255\007\000\069\000\072\000\247\255\235\255\246\255\
    \013\000\201\003\075\000\077\000\245\255\244\255\240\003\038\004\
    \111\004\184\004\212\004\219\004\024\005\078\000\079\000\242\255\
    \080\000\081\000\082\000\241\255\085\000\239\255\239\000\237\255\
    \238\255\086\000\091\000\234\255\014\000\158\000\233\255\092\000\
    \232\255\036\001\251\255\016\000\253\255\003\002\071\002\127\002\
    \252\255\237\000\164\000\148\000\194\000\255\255\213\000\241\000\
    \249\000\030\001\032\000\117\001\017\000\253\255\023\000\254\255\
    \073\001\057\002\126\001\158\001\156\000\255\255\018\000\190\001\
    \163\000\020\000\218\002\199\000\109\003\021\000\043\000\151\000\
    \166\003\254\255\000\000\003\004\063\001\106\004\252\255\001\002\
    \242\001\068\002\250\255\103\000\045\000\049\000\115\000\132\000\
    \255\255\190\003\075\000\253\255\074\000\252\255\251\255\172\000\
    \178\000\254\255\138\000\139\000\140\000\255\255\005\001\082\001\
    \253\255\131\001\255\255\254\255\093\004\248\255\141\000\043\003\
    \111\002\211\000\226\000\197\000\201\004\253\255\150\004\150\000\
    \151\000\249\255";
  Lexing.lex_backtrk =
   "\010\000\255\255\008\000\255\255\255\255\005\000\004\000\003\000\
    \002\000\001\000\010\000\000\000\255\255\255\255\001\000\001\000\
    \001\000\001\000\001\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \006\000\255\255\255\255\255\255\255\255\255\255\007\000\255\255\
    \255\255\009\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\024\000\022\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\000\000\255\255\001\000\255\255\255\255\
    \007\000\002\000\255\255\255\255\255\255\255\255\003\000\255\255\
    \255\255\004\000\255\255\255\255\005\000\255\255\255\255\006\000\
    \255\255\255\255\019\000\008\000\255\255\255\255\255\255\255\255\
    \009\000\024\000\010\000\255\255\255\255\255\255\024\000\024\000\
    \024\000\255\255\012\000\024\000\024\000\013\000\255\255\255\255\
    \014\000\016\000\255\255\255\255\255\255\255\255\015\000\255\255\
    \255\255\023\000\255\255\255\255\021\000\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\255\255\001\000\255\255\255\255\
    \255\255\001\000\255\255\255\255\255\255\255\255\001\000\255\255\
    \255\255\255\255\000\000\255\255\002\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\001\000\002\000\004\000\004\000\002\000\002\000\002\000\
    \002\000\255\255\004\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\006\000\255\255\007\000\007\000\007\000\007\000\007\000\
    \255\255\001\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \002\000\255\255\003\000\003\000\255\255\255\255\003\000\003\000\
    \255\255\002\000\255\255\255\255\000\000\255\255\004\000\255\255\
    \255\255\001\000\001\000\001\000\001\000\255\255\255\255\003\000\
    \005\000\255\255";
  Lexing.lex_default =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\000\000\255\255\000\000\255\255\000\000\
    \255\255\000\000\255\255\000\000\255\255\255\255\255\255\000\000\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\255\255\
    \255\255\255\255\255\255\000\000\255\255\000\000\000\000\049\000\
    \000\000\049\000\255\255\120\000\049\000\049\000\049\000\049\000\
    \049\000\049\000\049\000\049\000\049\000\049\000\049\000\049\000\
    \049\000\049\000\049\000\255\255\000\000\255\255\255\255\000\000\
    \255\255\255\255\255\255\000\000\255\255\000\000\255\255\255\255\
    \000\000\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\000\000\000\000\000\000\
    \255\255\049\000\255\255\255\255\000\000\000\000\049\000\049\000\
    \049\000\106\000\106\000\049\000\049\000\255\255\255\255\000\000\
    \255\255\255\255\255\255\000\000\255\255\000\000\118\000\000\000\
    \000\000\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \000\000\133\000\000\000\136\000\000\000\133\000\133\000\133\000\
    \000\000\138\000\138\000\139\000\139\000\000\000\143\000\143\000\
    \143\000\143\000\255\255\150\000\255\255\000\000\150\000\000\000\
    \150\000\150\000\150\000\150\000\150\000\000\000\255\255\150\000\
    \150\000\255\255\164\000\255\255\173\000\255\255\255\255\255\255\
    \173\000\000\000\255\255\173\000\255\255\173\000\000\000\255\255\
    \177\000\177\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\185\000\255\255\000\000\255\255\000\000\000\000\192\000\
    \192\000\000\000\255\255\255\255\255\255\000\000\199\000\199\000\
    \000\000\255\255\000\000\000\000\208\000\000\000\255\255\255\255\
    \208\000\255\255\255\255\255\255\208\000\000\000\255\255\255\255\
    \255\255\000\000";
  Lexing.lex_trans =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\010\000\012\000\012\000\031\000\011\000\037\000\037\000\
    \033\000\031\000\038\000\068\000\032\000\255\255\255\255\095\000\
    \123\000\255\255\255\255\149\000\157\000\255\255\151\000\166\000\
    \010\000\149\000\165\000\007\000\148\000\033\000\031\000\033\000\
    \007\000\032\000\008\000\000\000\119\000\004\000\028\000\029\000\
    \008\000\010\000\030\000\037\000\094\000\166\000\038\000\000\000\
    \165\000\000\000\005\000\006\000\033\000\009\000\169\000\041\000\
    \001\000\046\000\042\000\026\000\024\000\022\000\020\000\014\000\
    \010\000\019\000\013\000\007\000\015\000\016\000\017\000\013\000\
    \013\000\013\000\008\000\018\000\013\000\004\000\013\000\021\000\
    \023\000\025\000\027\000\033\000\034\000\040\000\044\000\043\000\
    \045\000\068\000\005\000\006\000\067\000\009\000\070\000\255\255\
    \001\000\039\000\255\255\125\000\123\000\071\000\076\000\124\000\
    \074\000\075\000\077\000\079\000\080\000\082\000\083\000\085\000\
    \086\000\088\000\064\000\003\000\002\000\035\000\036\000\089\000\
    \255\255\066\000\125\000\062\000\060\000\092\000\065\000\255\255\
    \093\000\255\255\255\255\099\000\255\255\100\000\110\000\111\000\
    \114\000\116\000\115\000\052\000\059\000\117\000\127\000\255\255\
    \255\255\255\255\255\255\126\000\128\000\146\000\255\255\121\000\
    \167\000\255\255\122\000\003\000\002\000\190\000\157\000\125\000\
    \123\000\158\000\188\000\124\000\186\000\151\000\255\255\185\000\
    \161\000\255\255\058\000\054\000\057\000\063\000\061\000\167\000\
    \255\255\255\255\255\255\255\255\255\255\055\000\125\000\056\000\
    \184\000\255\255\184\000\255\255\255\255\255\255\187\000\189\000\
    \193\000\196\000\197\000\113\000\255\255\215\000\211\000\255\255\
    \172\000\172\000\053\000\050\000\175\000\051\000\216\000\217\000\
    \255\255\255\255\255\255\255\255\000\000\210\000\000\000\255\255\
    \209\000\255\255\255\255\255\255\255\255\211\000\255\255\172\000\
    \255\255\174\000\194\000\255\255\210\000\000\000\174\000\209\000\
    \255\255\255\255\255\255\118\000\255\255\000\000\000\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\000\000\
    \000\000\255\255\184\000\255\255\000\000\037\000\255\255\031\000\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\140\000\255\255\255\255\255\255\255\255\255\255\149\000\
    \255\255\255\255\255\255\255\255\031\000\255\255\255\255\255\255\
    \139\000\255\255\000\000\000\000\000\000\000\000\000\000\200\000\
    \255\255\037\000\195\000\255\255\200\000\255\255\255\255\000\000\
    \255\255\255\255\255\255\255\255\000\000\255\255\000\000\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\255\255\141\000\
    \255\255\255\255\000\000\255\255\255\255\000\000\255\255\130\000\
    \172\000\172\000\000\000\000\000\175\000\000\000\255\255\255\255\
    \255\255\255\255\144\000\149\000\000\000\000\000\148\000\048\000\
    \000\000\112\000\255\255\000\000\255\255\255\255\000\000\172\000\
    \000\000\174\000\123\000\000\000\000\000\000\000\174\000\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\144\000\255\255\
    \109\000\255\255\255\255\255\255\255\255\000\000\145\000\000\000\
    \000\000\255\255\255\255\000\000\255\255\000\000\153\000\149\000\
    \255\255\134\000\148\000\201\000\000\000\000\000\000\000\255\255\
    \149\000\255\255\255\255\148\000\255\255\000\000\255\255\255\255\
    \255\255\255\255\255\255\000\000\255\255\153\000\000\000\255\255\
    \255\255\255\255\255\255\146\000\157\000\000\000\123\000\000\000\
    \132\000\000\000\131\000\151\000\255\255\203\000\000\000\255\255\
    \149\000\000\000\203\000\148\000\193\000\000\000\255\255\000\000\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\159\000\255\255\
    \149\000\000\000\000\000\148\000\000\000\000\000\000\000\000\000\
    \255\255\107\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \255\255\255\255\000\000\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\255\255\
    \000\000\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \000\000\255\255\152\000\000\000\255\255\255\255\255\255\255\255\
    \102\000\255\255\255\255\155\000\000\000\255\255\000\000\255\255\
    \255\255\203\000\255\255\000\000\000\000\202\000\000\000\000\000\
    \000\000\000\000\172\000\172\000\000\000\255\255\175\000\000\000\
    \255\255\000\000\000\000\255\255\255\255\255\255\255\255\101\000\
    \255\255\255\255\255\255\156\000\255\255\255\255\255\255\255\255\
    \255\255\172\000\255\255\174\000\255\255\000\000\255\255\000\000\
    \174\000\000\000\255\255\255\255\255\255\255\255\182\000\000\000\
    \179\000\000\000\000\000\000\000\000\000\255\255\255\255\000\000\
    \255\255\000\000\000\000\160\000\000\000\000\000\000\000\000\000\
    \097\000\095\000\153\000\149\000\096\000\000\000\148\000\000\000\
    \000\000\149\000\000\000\098\000\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\000\000\255\255\000\000\255\255\097\000\
    \255\255\153\000\255\255\000\000\000\000\255\255\000\000\000\000\
    \135\000\255\255\000\000\255\255\255\255\000\000\255\255\000\000\
    \000\000\000\000\255\255\255\255\255\255\181\000\255\255\180\000\
    \183\000\255\255\255\255\255\255\059\000\151\000\000\000\255\255\
    \255\255\255\255\255\255\000\000\255\255\000\000\149\000\255\255\
    \255\255\255\255\255\255\202\000\255\255\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\255\255\000\000\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\090\000\000\000\091\000\255\255\149\000\255\255\
    \000\000\000\000\255\255\000\000\132\000\000\000\000\000\255\255\
    \255\255\255\255\255\255\000\000\207\000\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\154\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\149\000\255\255\
    \000\000\255\255\255\255\255\255\000\000\255\255\255\255\000\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\255\255\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\167\000\166\000\000\000\000\000\165\000\
    \255\255\255\255\000\000\255\255\000\000\255\255\000\000\000\000\
    \255\255\000\000\178\000\000\000\255\255\000\000\084\000\255\255\
    \000\000\255\255\167\000\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\087\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\000\000\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\168\000\000\000\163\000\
    \170\000\000\000\000\000\000\000\000\000\000\000\255\255\255\255\
    \000\000\255\255\000\000\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\000\000\207\000\207\000\000\000\095\000\
    \214\000\149\000\255\255\255\255\255\255\255\255\000\000\000\000\
    \255\255\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\207\000\000\000\213\000\000\000\000\000\
    \000\000\000\000\213\000\000\000\000\000\000\000\171\000\000\000\
    \000\000\078\000\000\000\255\255\255\255\255\255\081\000\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\255\255\000\000\000\000\255\255\000\000\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\255\255\255\255\
    \000\000\000\000\255\255\255\255\255\255\000\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\255\255\072\000\000\000\073\000\255\255\000\000\000\000\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\255\255\000\000\255\255\255\255\255\255\000\000\
    \255\255\000\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\172\000\255\255\000\000\000\000\000\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\000\000\
    \255\255\255\255\255\255\255\255\255\255\069\000\000\000\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\097\000\095\000\000\000\000\000\096\000\000\000\
    \255\255\255\255\169\000\255\255\000\000\000\000\255\255\000\000\
    \000\000\168\000\000\000\172\000\255\255\000\000\000\000\000\000\
    \000\000\097\000\000\000\000\000\255\255\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\255\255\255\255\000\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \255\255\000\000\000\000\255\255\255\255\255\255\255\255\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\255\255\255\255\000\000\255\255\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\000\000\255\255\000\000\000\000\255\255\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\000\000\000\000\
    \172\000\255\255\000\000\000\000\255\255\255\255\000\000\255\255\
    \000\000\255\255\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\255\255\000\000\
    \255\255\000\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \255\255\255\255\255\255\255\255\103\000\000\000\211\000\210\000\
    \000\000\000\000\209\000\255\255\255\255\255\255\255\255\000\000\
    \000\000\000\000\000\000\255\255\255\255\000\000\000\000\255\255\
    \000\000\255\255\000\000\000\000\255\255\211\000\000\000\000\000\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \000\000\000\000\255\255\000\000\255\255\000\000\255\255\000\000\
    \000\000\000\000\255\255\255\255\000\000\000\000\104\000\000\000\
    \212\000\255\255\207\000\255\255\255\255\206\000\255\255\207\000\
    \207\000\255\255\255\255\214\000\255\255\000\000\255\255\172\000\
    \255\255\105\000\255\255\255\255\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\207\000\000\000\
    \213\000\000\000\000\000\000\000\000\000\213\000\255\255\000\000\
    \000\000\255\255\255\255\000\000\000\000\255\255\000\000\000\000\
    \000\000\095\000\255\255\255\255\255\255\255\255\255\255\000\000\
    \000\000\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\105\000\105\000\000\000\000\000\255\255\255\255\105\000\
    \000\000\255\255\000\000\000\000\105\000\255\255\105\000\000\000\
    \255\255\255\255\255\255\255\255\000\000\255\255\000\000\000\000\
    \255\255\000\000\105\000\105\000\255\255\105\000\105\000\105\000\
    \000\000\000\000\000\000\105\000\000\000\000\000\255\255\000\000\
    \105\000\000\000\105\000\255\255\212\000\255\255\207\000\255\255\
    \255\255\255\255\255\255\000\000\000\000\000\000\105\000\105\000\
    \000\000\000\000\000\000\105\000\000\000\255\255\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\255\255\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\000\000\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \000\000\000\000\000\000\108\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\105\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\000\000\000\000\000\000\205\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\000\000\255\255\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\255\255\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255";
  Lexing.lex_check =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\011\000\032\000\000\000\038\000\040\000\
    \033\000\033\000\040\000\067\000\033\000\051\000\051\000\096\000\
    \124\000\051\000\131\000\148\000\158\000\131\000\161\000\165\000\
    \000\000\150\000\165\000\000\000\150\000\030\000\030\000\033\000\
    \007\000\030\000\000\000\255\255\051\000\000\000\004\000\028\000\
    \008\000\010\000\029\000\036\000\090\000\166\000\036\000\255\255\
    \166\000\255\255\000\000\000\000\030\000\000\000\170\000\002\000\
    \000\000\001\000\002\000\005\000\006\000\007\000\008\000\009\000\
    \010\000\013\000\009\000\010\000\014\000\015\000\016\000\014\000\
    \015\000\016\000\010\000\017\000\018\000\010\000\017\000\020\000\
    \022\000\024\000\026\000\034\000\030\000\039\000\041\000\042\000\
    \044\000\047\000\010\000\010\000\047\000\010\000\069\000\049\000\
    \010\000\036\000\049\000\050\000\050\000\070\000\072\000\050\000\
    \073\000\074\000\076\000\078\000\079\000\081\000\082\000\084\000\
    \085\000\087\000\047\000\000\000\000\000\003\000\035\000\088\000\
    \049\000\047\000\050\000\047\000\047\000\091\000\047\000\049\000\
    \092\000\049\000\049\000\098\000\049\000\099\000\109\000\110\000\
    \112\000\113\000\114\000\047\000\047\000\116\000\121\000\049\000\
    \049\000\049\000\049\000\122\000\127\000\146\000\139\000\050\000\
    \167\000\139\000\050\000\010\000\010\000\179\000\156\000\125\000\
    \125\000\156\000\180\000\125\000\181\000\160\000\138\000\182\000\
    \160\000\138\000\047\000\047\000\047\000\047\000\047\000\167\000\
    \049\000\049\000\049\000\049\000\049\000\047\000\125\000\047\000\
    \183\000\052\000\183\000\049\000\052\000\049\000\186\000\188\000\
    \194\000\195\000\196\000\112\000\140\000\206\000\211\000\140\000\
    \163\000\163\000\047\000\047\000\163\000\047\000\215\000\216\000\
    \049\000\049\000\052\000\049\000\255\255\209\000\255\255\142\000\
    \209\000\052\000\142\000\052\000\052\000\211\000\052\000\163\000\
    \053\000\163\000\191\000\053\000\210\000\255\255\163\000\210\000\
    \192\000\052\000\052\000\052\000\052\000\255\255\255\255\137\000\
    \118\000\118\000\137\000\143\000\118\000\255\255\143\000\255\255\
    \255\255\053\000\183\000\144\000\255\255\040\000\144\000\033\000\
    \053\000\255\255\053\000\053\000\051\000\053\000\255\255\118\000\
    \131\000\139\000\052\000\052\000\052\000\052\000\052\000\150\000\
    \053\000\053\000\053\000\053\000\030\000\052\000\054\000\052\000\
    \138\000\054\000\255\255\255\255\255\255\255\255\255\255\198\000\
    \145\000\036\000\191\000\145\000\198\000\118\000\129\000\255\255\
    \192\000\129\000\052\000\052\000\255\255\052\000\255\255\054\000\
    \255\255\053\000\053\000\053\000\053\000\053\000\054\000\140\000\
    \054\000\054\000\255\255\054\000\053\000\255\255\053\000\129\000\
    \172\000\172\000\255\255\255\255\172\000\255\255\054\000\054\000\
    \054\000\054\000\142\000\152\000\255\255\255\255\152\000\047\000\
    \255\255\053\000\053\000\255\255\053\000\049\000\255\255\172\000\
    \255\255\172\000\050\000\255\255\255\255\255\255\172\000\055\000\
    \137\000\137\000\055\000\118\000\255\255\255\255\143\000\054\000\
    \054\000\054\000\054\000\054\000\199\000\255\255\144\000\255\255\
    \255\255\199\000\054\000\255\255\054\000\255\255\147\000\147\000\
    \055\000\129\000\147\000\198\000\255\255\255\255\255\255\055\000\
    \154\000\055\000\055\000\154\000\055\000\255\255\056\000\054\000\
    \054\000\056\000\054\000\255\255\139\000\147\000\255\255\055\000\
    \055\000\055\000\055\000\145\000\156\000\255\255\125\000\255\255\
    \129\000\255\255\129\000\160\000\138\000\201\000\255\255\056\000\
    \155\000\255\255\201\000\155\000\191\000\255\255\056\000\255\255\
    \056\000\056\000\192\000\056\000\255\255\255\255\255\255\052\000\
    \055\000\055\000\055\000\055\000\055\000\255\255\056\000\056\000\
    \056\000\056\000\140\000\055\000\057\000\055\000\152\000\057\000\
    \159\000\255\255\255\255\159\000\255\255\255\255\255\255\255\255\
    \199\000\055\000\255\255\255\255\255\255\142\000\255\255\255\255\
    \055\000\055\000\255\255\055\000\255\255\057\000\053\000\056\000\
    \056\000\056\000\056\000\056\000\057\000\255\255\057\000\057\000\
    \255\255\057\000\056\000\255\255\056\000\137\000\255\255\118\000\
    \255\255\143\000\147\000\255\255\057\000\057\000\057\000\057\000\
    \056\000\144\000\058\000\154\000\255\255\058\000\255\255\056\000\
    \056\000\201\000\056\000\255\255\255\255\198\000\255\255\255\255\
    \255\255\255\255\175\000\175\000\255\255\133\000\175\000\255\255\
    \133\000\255\255\255\255\058\000\054\000\057\000\057\000\057\000\
    \057\000\057\000\058\000\155\000\058\000\058\000\145\000\058\000\
    \057\000\175\000\057\000\175\000\129\000\255\255\133\000\255\255\
    \175\000\255\255\058\000\058\000\058\000\058\000\176\000\255\255\
    \176\000\255\255\255\255\255\255\255\255\057\000\057\000\255\255\
    \057\000\255\255\255\255\159\000\255\255\255\255\255\255\255\255\
    \059\000\059\000\153\000\153\000\059\000\255\255\153\000\255\255\
    \255\255\152\000\255\255\058\000\058\000\058\000\058\000\058\000\
    \255\255\134\000\199\000\255\255\134\000\255\255\058\000\059\000\
    \058\000\153\000\059\000\255\255\255\255\055\000\255\255\255\255\
    \133\000\059\000\255\255\059\000\059\000\255\255\059\000\255\255\
    \255\255\255\255\134\000\058\000\058\000\176\000\058\000\176\000\
    \176\000\059\000\059\000\059\000\059\000\147\000\255\255\060\000\
    \208\000\208\000\060\000\255\255\208\000\255\255\154\000\133\000\
    \177\000\133\000\177\000\201\000\056\000\255\255\255\255\255\255\
    \255\255\135\000\255\255\255\255\135\000\255\255\255\255\208\000\
    \060\000\255\255\059\000\059\000\059\000\059\000\059\000\060\000\
    \255\255\060\000\060\000\255\255\060\000\059\000\155\000\059\000\
    \255\255\255\255\135\000\255\255\134\000\255\255\255\255\060\000\
    \060\000\060\000\060\000\255\255\208\000\061\000\255\255\208\000\
    \061\000\255\255\059\000\059\000\255\255\059\000\153\000\255\255\
    \255\255\255\255\057\000\255\255\255\255\255\255\159\000\177\000\
    \255\255\177\000\177\000\134\000\255\255\134\000\061\000\255\255\
    \060\000\060\000\060\000\060\000\060\000\061\000\255\255\061\000\
    \061\000\255\255\061\000\060\000\062\000\060\000\255\255\062\000\
    \255\255\255\255\255\255\255\255\135\000\061\000\061\000\061\000\
    \061\000\255\255\255\255\162\000\162\000\255\255\255\255\162\000\
    \060\000\060\000\255\255\060\000\255\255\062\000\255\255\255\255\
    \058\000\255\255\176\000\255\255\062\000\255\255\062\000\062\000\
    \255\255\062\000\162\000\135\000\255\255\135\000\061\000\061\000\
    \061\000\061\000\061\000\133\000\062\000\062\000\062\000\062\000\
    \255\255\061\000\063\000\061\000\255\255\063\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\162\000\255\255\162\000\
    \162\000\255\255\255\255\255\255\255\255\255\255\061\000\061\000\
    \255\255\061\000\255\255\063\000\255\255\062\000\062\000\062\000\
    \062\000\062\000\063\000\255\255\063\000\063\000\255\255\063\000\
    \062\000\255\255\062\000\255\255\207\000\207\000\255\255\059\000\
    \207\000\153\000\063\000\063\000\063\000\063\000\255\255\255\255\
    \064\000\255\255\255\255\064\000\177\000\062\000\062\000\134\000\
    \062\000\255\255\255\255\207\000\255\255\207\000\255\255\255\255\
    \255\255\255\255\207\000\255\255\255\255\255\255\162\000\255\255\
    \255\255\064\000\255\255\063\000\063\000\063\000\063\000\063\000\
    \064\000\255\255\064\000\064\000\255\255\064\000\063\000\065\000\
    \063\000\255\255\065\000\255\255\255\255\060\000\255\255\208\000\
    \064\000\064\000\064\000\064\000\255\255\255\255\164\000\164\000\
    \255\255\255\255\164\000\063\000\063\000\255\255\063\000\135\000\
    \065\000\255\255\255\255\255\255\255\255\255\255\255\255\065\000\
    \255\255\065\000\065\000\255\255\065\000\164\000\255\255\255\255\
    \255\255\064\000\064\000\064\000\064\000\064\000\255\255\065\000\
    \065\000\065\000\065\000\255\255\064\000\066\000\064\000\255\255\
    \066\000\255\255\255\255\061\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\164\000\164\000\255\255\255\255\255\255\168\000\
    \168\000\064\000\064\000\168\000\064\000\255\255\066\000\255\255\
    \065\000\065\000\065\000\065\000\065\000\066\000\255\255\066\000\
    \066\000\255\255\066\000\065\000\255\255\065\000\168\000\185\000\
    \185\000\255\255\062\000\185\000\255\255\066\000\066\000\066\000\
    \066\000\255\255\097\000\097\000\255\255\255\255\097\000\255\255\
    \065\000\065\000\162\000\065\000\255\255\255\255\185\000\255\255\
    \255\255\168\000\255\255\168\000\168\000\255\255\255\255\255\255\
    \255\255\097\000\255\255\255\255\097\000\255\255\066\000\066\000\
    \066\000\066\000\066\000\097\000\255\255\097\000\097\000\255\255\
    \097\000\066\000\102\000\066\000\185\000\102\000\255\255\255\255\
    \063\000\255\255\255\255\097\000\097\000\097\000\097\000\255\255\
    \255\255\255\255\255\255\255\255\171\000\171\000\066\000\066\000\
    \171\000\066\000\255\255\102\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\102\000\255\255\102\000\102\000\255\255\102\000\
    \255\255\255\255\255\255\171\000\097\000\097\000\097\000\097\000\
    \097\000\255\255\102\000\102\000\102\000\102\000\255\255\097\000\
    \103\000\097\000\255\255\103\000\255\255\255\255\064\000\255\255\
    \255\255\255\255\185\000\255\255\255\255\255\255\255\255\255\255\
    \171\000\171\000\255\255\255\255\097\000\097\000\255\255\097\000\
    \255\255\103\000\255\255\102\000\102\000\102\000\102\000\102\000\
    \103\000\255\255\103\000\103\000\255\255\103\000\102\000\255\255\
    \102\000\255\255\255\255\255\255\255\255\065\000\255\255\255\255\
    \103\000\103\000\103\000\103\000\102\000\255\255\204\000\204\000\
    \255\255\255\255\204\000\102\000\102\000\164\000\102\000\255\255\
    \255\255\255\255\255\255\173\000\173\000\255\255\255\255\173\000\
    \255\255\104\000\255\255\255\255\104\000\204\000\255\255\255\255\
    \255\255\103\000\103\000\103\000\103\000\103\000\255\255\255\255\
    \255\255\255\255\173\000\255\255\103\000\255\255\103\000\255\255\
    \255\255\255\255\104\000\066\000\255\255\255\255\103\000\255\255\
    \204\000\104\000\204\000\104\000\104\000\204\000\104\000\214\000\
    \214\000\103\000\103\000\214\000\103\000\255\255\168\000\173\000\
    \173\000\104\000\104\000\104\000\104\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\214\000\255\255\
    \214\000\255\255\255\255\255\255\255\255\214\000\185\000\255\255\
    \255\255\105\000\105\000\255\255\255\255\105\000\255\255\255\255\
    \255\255\097\000\104\000\104\000\104\000\104\000\104\000\255\255\
    \255\255\255\255\212\000\212\000\255\255\104\000\212\000\104\000\
    \105\000\105\000\105\000\255\255\255\255\106\000\106\000\105\000\
    \255\255\106\000\255\255\255\255\105\000\107\000\105\000\255\255\
    \107\000\212\000\104\000\104\000\255\255\104\000\255\255\255\255\
    \102\000\255\255\105\000\105\000\106\000\106\000\106\000\105\000\
    \255\255\255\255\255\255\106\000\255\255\255\255\107\000\255\255\
    \106\000\255\255\106\000\171\000\212\000\107\000\212\000\107\000\
    \107\000\212\000\107\000\255\255\255\255\255\255\106\000\106\000\
    \255\255\255\255\255\255\106\000\255\255\107\000\107\000\107\000\
    \107\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\108\000\255\255\255\255\108\000\103\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\107\000\107\000\
    \107\000\107\000\107\000\108\000\255\255\255\255\255\255\255\255\
    \255\255\107\000\108\000\107\000\108\000\108\000\255\255\108\000\
    \255\255\255\255\255\255\107\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\108\000\108\000\108\000\108\000\107\000\107\000\
    \255\255\107\000\255\255\255\255\255\255\204\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\173\000\255\255\255\255\255\255\255\255\104\000\
    \255\255\255\255\255\255\108\000\108\000\108\000\108\000\108\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\108\000\255\255\
    \108\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\108\000\108\000\255\255\108\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \105\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\212\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\106\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\107\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \108\000";
  Lexing.lex_base_code =
   "\000\000\000\000\001\000\000\000\000\000\000\000\000\000\001\000\
    \001\000\002\000\006\000\000\000\000\000\000\000\005\000\007\000\
    \008\000\009\000\000\000\043\000\000\000\040\000\000\000\037\000\
    \000\000\034\000\000\000\031\000\000\000\000\000\000\000\051\000\
    \000\000\000\000\007\000\000\000\000\000\064\000\000\000\008\000\
    \000\000\000\000\000\000\025\000\000\000\072\000\001\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_backtrk_code =
   "\001\000\000\000\025\000\000\000\000\000\031\000\034\000\037\000\
    \040\000\043\000\001\000\000\000\000\000\000\000\043\000\043\000\
    \043\000\043\000\043\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \051\000\000\000\000\000\000\000\000\000\000\000\064\000\000\000\
    \000\000\072\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_default_code =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_trans_code =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\004\000\000\000\000\000\000\000\000\000\000\000\004\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \004\000\000\000\000\000\007\000\007\000\000\000\004\000\000\000\
    \000\000\007\000\010\000\010\000\000\000\046\000\000\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\013\000\016\000\000\000\019\000\028\000\019\000\
    \013\000\016\000\019\000\019\000\019\000\019\000\019\000\056\000\
    \069\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\059\000\022\000\000\000\000\000\000\000\
    \000\000\000\000\022\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_check_code =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\255\255\010\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\000\000\007\000\255\255\010\000\255\255\
    \255\255\010\000\000\000\008\000\255\255\029\000\255\255\255\255\
    \010\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\255\255\000\000\002\000\009\000\
    \010\000\010\000\014\000\010\000\015\000\016\000\017\000\034\000\
    \039\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\035\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\010\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255";
  Lexing.lex_code =
   "\255\000\002\255\002\255\255\006\255\255\007\255\255\004\255\255\
    \005\255\255\008\255\255\003\255\255\000\003\255\013\255\255\000\
    \004\255\000\005\255\000\006\255\000\007\255\000\008\255\010\255\
    \009\255\255\000\009\001\010\255\010\255\255\012\255\011\255\255\
    \000\011\001\012\255\012\255\255\000\013\255";
}

let rec parse_bol c lexbuf =
  lexbuf.Lexing.lex_mem <- Array.make 14 (-1);(* L=1 [2] <- p ;  *)
  lexbuf.Lexing.lex_mem.(2) <- lexbuf.Lexing.lex_curr_pos ;
 __ocaml_lex_parse_bol_rec c lexbuf 0
and __ocaml_lex_parse_bol_rec c lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 473 "wikicreole.mll"
               (
      end_paragraph c 0;
      parse_bol c lexbuf
    )
# 1119 "wikicreole.ml"

  | 1 ->
let
# 478 "wikicreole.mll"
                   att
# 1125 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 478 "wikicreole.mll"
                        (
      end_paragraph c 0;
      let l = count '=' (Lexing.lexeme lexbuf) in
      end_section c l;
      push_section c l;
      assert (match c.stack with Paragraph _ -> true | _ -> false);
      c.stack <- Heading (l, read_attribs att parse_attribs c lexbuf);
      c.heading <- true;
      parse_rem c lexbuf
    )
# 1138 "wikicreole.ml"

  | 2 ->
let
# 488 "wikicreole.mll"
                                     att
# 1144 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 488 "wikicreole.mll"
                                          (
      let lev = count '*' (Lexing.lexeme lexbuf) in
      if not (start_list_item c Unordered lev att parse_attribs lexbuf)
      then begin
        let s = Lexing.lexeme lexbuf in
        let l = String.index s '*' in
        if l > 0 then push_string c (String.sub s 0 l);
        for i = 1 to lev / 2 - 1 do
          style_change c Bold "" parse_attribs lexbuf
        done;
        if lev land 1 = 1
        then begin
          style_change c Bold "" parse_attribs lexbuf;
          push_string c "*";
          push_string c att;
        end
        else
          style_change c Bold att parse_attribs lexbuf
      end;
      parse_rem c lexbuf
    )
# 1168 "wikicreole.ml"

  | 3 ->
let
# 509 "wikicreole.mll"
                                     att
# 1174 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 509 "wikicreole.mll"
                                          (
      let lev = count '#' (Lexing.lexeme lexbuf) in
      if not (start_list_item c Ordered lev att parse_attribs lexbuf)
      then begin
        let s = Lexing.lexeme lexbuf in
        let l = String.index s '#' in
        if l > 0 then push_string c (String.sub s 0 l);
        for i = 1 to lev / 2 - 1 do
          style_change c Monospace "" parse_attribs lexbuf
        done;
        if lev land 1 = 1
        then begin
          style_change c Monospace "" parse_attribs lexbuf;
          push_string c "#";
          push_string c att;
        end
        else
          style_change c Bold att parse_attribs lexbuf
      end;
      parse_rem c lexbuf
    )
# 1198 "wikicreole.ml"

  | 4 ->
let
# 530 "wikicreole.mll"
                                   att
# 1204 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 530 "wikicreole.mll"
                                        (
      let (list_attribs, item_attribs) =
        read_list_attribs att parse_attribs c lexbuf
      in
      if close_descr_entry c
      then c.stack <- Descr_title (item_attribs, c.stack)
      else begin
        end_paragraph c 0;
        c.stack <- Descr_title (item_attribs, Descr (list_attribs, c.stack));
        c.descr <- []
      end;
      parse_rem c lexbuf
    )
# 1220 "wikicreole.ml"

  | 5 ->
let
# 543 "wikicreole.mll"
                                   att
# 1226 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 543 "wikicreole.mll"
                                        (
      let (list_attribs, item_attribs) =
        read_list_attribs att parse_attribs c lexbuf
      in
      if close_descr_entry c
      then c.stack <- Descr_def (item_attribs, c.stack)
      else begin
        end_paragraph c 0;
        c.stack <- Descr_def (item_attribs, Descr (list_attribs, c.stack));
        c.descr <- []
      end;
      parse_rem c lexbuf
    )
# 1242 "wikicreole.ml"

  | 6 ->
let
# 556 "wikicreole.mll"
                                      att
# 1248 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_mem.(1) in
# 556 "wikicreole.mll"
                                                                            (
      end_paragraph c 0;
      push_flow c
	(B.flow
	   (B.hr_elem
              (read_attribs att parse_attribs c lexbuf)));
      parse_bol c lexbuf
    )
# 1259 "wikicreole.ml"

  | 7 ->
let
# 564 "wikicreole.mll"
                                     att
# 1265 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_mem.(1) in
# 564 "wikicreole.mll"
                                                             (
      end_paragraph c 0;
      parse_nowiki c (read_attribs att parse_attribs c lexbuf) lexbuf
    )
# 1272 "wikicreole.ml"

  | 8 ->
let
# 568 "wikicreole.mll"
                                   att
# 1278 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 568 "wikicreole.mll"
                                        (
      start_table_row c false
        (read_table_attribs att parse_attribs c lexbuf);
      parse_rem c lexbuf
    )
# 1286 "wikicreole.ml"

  | 9 ->
let
# 573 "wikicreole.mll"
                                    att
# 1292 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 573 "wikicreole.mll"
                                         (
      start_table_row c true
        (read_table_attribs att parse_attribs c lexbuf);
      parse_rem c lexbuf
    )
# 1300 "wikicreole.ml"

  | 10 ->
let
# 578 "wikicreole.mll"
                               att
# 1306 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 578 "wikicreole.mll"
                                    (
      let attribs = read_attribs att parse_attribs c lexbuf in
      if attribs <> []
      then
	(match c.stack with
           | Paragraph _ -> c.stack <- Paragraph attribs
           | _ -> ());
      parse_rem c lexbuf
    )
# 1318 "wikicreole.ml"

  | 11 ->
# 587 "wikicreole.mll"
       (
      parse_rem c lexbuf
    )
# 1325 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_bol_rec c lexbuf __ocaml_lex_state

and parse_rem c lexbuf =
   __ocaml_lex_parse_rem_rec c lexbuf 47
and __ocaml_lex_parse_rem_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 593 "wikicreole.mll"
               (
      (* Headings are single lines *)
      if c.heading then
        end_paragraph c 0
      else
        push_chars c lexbuf;
      parse_bol c lexbuf
    )
# 1344 "wikicreole.ml"

  | 1 ->
let
# 601 "wikicreole.mll"
                      att
# 1350 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 601 "wikicreole.mll"
                           (
      style_change c Bold att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1357 "wikicreole.ml"

  | 2 ->
let
# 605 "wikicreole.mll"
                      att
# 1363 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 605 "wikicreole.mll"
                           (
      style_change c Italic att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1370 "wikicreole.ml"

  | 3 ->
let
# 609 "wikicreole.mll"
                      att
# 1376 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 609 "wikicreole.mll"
                           (
      style_change c Monospace att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1383 "wikicreole.ml"

  | 4 ->
let
# 613 "wikicreole.mll"
                      att
# 1389 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 613 "wikicreole.mll"
                           (
      style_change c Superscripted att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1396 "wikicreole.ml"

  | 5 ->
let
# 617 "wikicreole.mll"
                      att
# 1402 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 617 "wikicreole.mll"
                           (
      style_change c Subscripted att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1409 "wikicreole.ml"

  | 6 ->
let
# 621 "wikicreole.mll"
                      att
# 1415 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 621 "wikicreole.mll"
                           (
      style_change c Underlined att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1422 "wikicreole.ml"

  | 7 ->
let
# 625 "wikicreole.mll"
                      att
# 1428 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 625 "wikicreole.mll"
                           (
      style_change c Linethrough att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1435 "wikicreole.ml"

  | 8 ->
let
# 629 "wikicreole.mll"
                      att
# 1441 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 629 "wikicreole.mll"
                           (
      style_change c Linethrough att parse_attribs lexbuf;
      parse_rem c lexbuf
    )
# 1448 "wikicreole.ml"

  | 9 ->
# 633 "wikicreole.mll"
                                           (
      if c.heading then
        end_paragraph c 0
      else
        push_chars c lexbuf;
      parse_bol c lexbuf
    )
# 1459 "wikicreole.ml"

  | 10 ->
let
# 640 "wikicreole.mll"
                      att
# 1465 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 641 "wikicreole.mll"
      ( parse_link (Lexing.lexeme_start lexbuf) "" None c (read_attribs att parse_attribs c lexbuf) lexbuf )
# 1469 "wikicreole.ml"

  | 11 ->
# 642 "wikicreole.mll"
         (
      begin match c.stack with
        Link (addr, attribs, stack) ->
          pop_link c addr attribs stack
      | _ ->
          push_chars c lexbuf
      end;
      parse_rem c lexbuf
    )
# 1482 "wikicreole.ml"

  | 12 ->
# 652 "wikicreole.mll"
                                                 (
      if c.link then
        push_chars c lexbuf
      else
	let addr_string = Lexing.lexeme lexbuf in
	let addr = B.make_href c.param addr_string None in
        c.phrasing_mix <-
          B.a_elem_phrasing [] addr [B.chars addr_string] :: c.phrasing_mix;
      parse_rem c lexbuf
  )
# 1496 "wikicreole.ml"

  | 13 ->
let
# 662 "wikicreole.mll"
                        att
# 1502 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 662 "wikicreole.mll"
                             (
      push c (B.br_elem (read_attribs att parse_attribs c lexbuf));
      parse_rem c lexbuf
    )
# 1509 "wikicreole.ml"

  | 14 ->
let
# 666 "wikicreole.mll"
                      att
# 1515 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 667 "wikicreole.mll"
      ( parse_image c (read_attribs att parse_attribs c lexbuf) lexbuf )
# 1519 "wikicreole.ml"

  | 15 ->
# 668 "wikicreole.mll"
                                                        (
      let s = Lexing.lexeme lexbuf in
      let l = String.length s in
      let name = String.sub s 2 (l - 2) in
      let start = Lexing.lexeme_start lexbuf in
      let (wiki_content, ext_info) = B.plugin name in
      let content, args =
        parse_extension start name wiki_content [] c lexbuf in
      B.plugin_action name start (Lexing.lexeme_end lexbuf)
        c.param (List.rev args) content;
      match ext_info c.param args content with
      | `Phrasing_without_interactive i ->
          push c i;
          parse_rem c lexbuf
      | `Phrasing_link (addr, attribs, content) ->
          c.link_content <- [ content ];
          pop_link c addr attribs c.stack;
          parse_rem c lexbuf
      | `Flow5 b ->
          end_paragraph c 0;
          push_flow c b;
          parse_bol c lexbuf
      | `Flow5_link (addr, attribs, content) ->
          end_paragraph c 0;
	  push_flow c (B.a_elem_flow attribs addr [content]);
          parse_bol c lexbuf
    )
# 1550 "wikicreole.ml"

  | 16 ->
let
# 695 "wikicreole.mll"
                       att
# 1556 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 3) lexbuf.Lexing.lex_curr_pos in
# 696 "wikicreole.mll"
      ( parse_tt c (read_attribs att parse_attribs c lexbuf) lexbuf )
# 1560 "wikicreole.ml"

  | 17 ->
# 697 "wikicreole.mll"
                                       (
      let s = Lexing.lexeme lexbuf in
      (* It amounts to the same to quote a UTF-8 char or its first byte *)
      push_string c (String.sub s 1 1);
      parse_rem c lexbuf
    )
# 1570 "wikicreole.ml"

  | 18 ->
# 703 "wikicreole.mll"
         (
      push c B.nbsp;
      parse_rem c lexbuf
    )
# 1578 "wikicreole.ml"

  | 19 ->
# 707 "wikicreole.mll"
         (
      push c B.endash;
      parse_rem c lexbuf
    )
# 1586 "wikicreole.ml"

  | 20 ->
# 711 "wikicreole.mll"
          (
      push c B.emdash;
      parse_rem c lexbuf
    )
# 1594 "wikicreole.ml"

  | 21 ->
# 715 "wikicreole.mll"
                                        (
      (match c.stack with
         | Entry _ | Row _ | Table _ -> ()
             (* we skip ending | if in table row *)
         | _ -> push_chars c lexbuf);
      parse_bol c lexbuf
    )
# 1605 "wikicreole.ml"

  | 22 ->
let
# 722 "wikicreole.mll"
                     att
# 1611 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 722 "wikicreole.mll"
                          (
      if close_entry c then
        c.stack <- Entry (false,
                          read_attribs att parse_attribs c lexbuf,
                          c.stack)
      else
        push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 1623 "wikicreole.ml"

  | 23 ->
let
# 731 "wikicreole.mll"
                      att
# 1629 "wikicreole.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 2) lexbuf.Lexing.lex_curr_pos in
# 731 "wikicreole.mll"
                           (
      if close_entry c then
        c.stack <- Entry (true,
                          read_attribs att parse_attribs c lexbuf,
                          c.stack)
      else
        push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 1641 "wikicreole.ml"

  | 24 ->
# 740 "wikicreole.mll"
                                  (
      push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 1649 "wikicreole.ml"

  | 25 ->
# 744 "wikicreole.mll"
      (
     Ocsigen_messages.warning
       ("Wikicreole: Unrecognized char "^(Lexing.lexeme lexbuf)^".");
     raise Unrecognized_char
  )
# 1658 "wikicreole.ml"

  | 26 ->
# 749 "wikicreole.mll"
        (
      end_paragraph c 0;
      end_section c 1
    )
# 1666 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_rem_rec c lexbuf __ocaml_lex_state

and parse_link beg begaddr fragment c attribs lexbuf =
   __ocaml_lex_parse_link_rec beg begaddr fragment c attribs lexbuf 129
and __ocaml_lex_parse_link_rec beg begaddr fragment c attribs lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 756 "wikicreole.mll"
                                                 (
      push_string c "[["; (*VVV We loose attributes *)
      push_string c begaddr;
      (match fragment with
         | None -> ()
         | Some f ->
             push_string c "#";
             push_string c f);
      parse_rem c lexbuf
    )
# 1687 "wikicreole.ml"

  | 1 ->
# 767 "wikicreole.mll"
          ( match fragment with
              | None ->
                  parse_link beg (begaddr^Lexing.lexeme lexbuf) None c attribs lexbuf
              | Some f ->
                  parse_link beg begaddr (Some (f^Lexing.lexeme lexbuf)) c attribs lexbuf
          )
# 1697 "wikicreole.ml"

  | 2 ->
# 773 "wikicreole.mll"
               (
      let lb = Lexing.lexeme lexbuf in
      if c.link then begin
        push_string c "[["; (*VVV We loose attributes *)
        push_string c begaddr;
        (match fragment with
          | None -> ()
          | Some f ->
              push_string c "#";
              push_string c f);
        push_string c lb
      end
      else begin
        B.link_action begaddr fragment attribs
          (beg+2, Lexing.lexeme_start lexbuf) c.param;
        let addr = B.make_href c.param begaddr fragment in
        if lb = "|"
        then begin
          c.stack <- Link (addr, attribs, c.stack);
          c.link <- true
        end
        else
          let text = match fragment with
            | None -> begaddr
            | Some f -> begaddr^"#"^f
          in
          c.phrasing_mix <-
            B.a_elem_phrasing
            attribs addr [B.chars text] :: c.phrasing_mix;
      end;
      parse_rem c lexbuf
  )
# 1733 "wikicreole.ml"

  | 3 ->
# 805 "wikicreole.mll"
                       (
      let s = Lexing.lexeme lexbuf in
      let char = String.sub s 1 1 in
      let char =
        if char = "]" || char = "|" || char = "#" || char = "~"
        then char
        else "~"^char
      in
      (* It amounts to the same to quote a UTF-8 char or its first byte *)
       match fragment with
         | None -> parse_link beg (begaddr^char) fragment c attribs lexbuf
         | Some f -> parse_link beg begaddr (Some (f^char)) c attribs lexbuf
    )
# 1750 "wikicreole.ml"

  | 4 ->
# 818 "wikicreole.mll"
        ( let begaddr = match fragment with
            | None -> begaddr
            | Some f -> begaddr^"#"^f
          in parse_link beg begaddr (Some "") c attribs lexbuf )
# 1758 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_link_rec beg begaddr fragment c attribs lexbuf __ocaml_lex_state

and parse_image c attribs lexbuf =
   __ocaml_lex_parse_image_rec c attribs lexbuf 137
and __ocaml_lex_parse_image_rec c attribs lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 827 "wikicreole.mll"
                                               (
      let s = Lexing.lexeme lexbuf in
      let i = String.index s '|' in
      let url = B.make_href c.param (String.sub s 0 i) None in
      let alt = String.sub s (i + 1) (String.length s - i - 3) in
      push c (B.img_elem attribs url alt);
      parse_rem c lexbuf
    )
# 1777 "wikicreole.ml"

  | 1 ->
# 835 "wikicreole.mll"
       (
      push_string c "{{"; (*VVV We loose attributes *)
      parse_rem c lexbuf
    )
# 1785 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_image_rec c attribs lexbuf __ocaml_lex_state

and parse_tt c attribs lexbuf =
   __ocaml_lex_parse_tt_rec c attribs lexbuf 142
and __ocaml_lex_parse_tt_rec c attribs lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 842 "wikicreole.mll"
                                                           (
      let s = Lexing.lexeme lexbuf in
      let txt = String.sub s 0 (String.length s - 3) in
      push c (B.tt_elem attribs [B.phrasing (B.chars txt)]);
      parse_rem c lexbuf
    )
# 1802 "wikicreole.ml"

  | 1 ->
# 848 "wikicreole.mll"
       (
      push_string c "{{{"; (*VVV We loose attributes *)
      parse_rem c lexbuf
    )
# 1810 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_tt_rec c attribs lexbuf __ocaml_lex_state

and parse_nowiki c attribs lexbuf =
   __ocaml_lex_parse_nowiki_rec c attribs lexbuf 147
and __ocaml_lex_parse_nowiki_rec c attribs lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 855 "wikicreole.mll"
                                           (
      let s = Lexing.lexeme lexbuf in
      c.pre_content <- String.sub s 1 (String.length s - 1) :: c.pre_content;
      parse_nowiki c attribs lexbuf
    )
# 1826 "wikicreole.ml"

  | 1 ->
# 860 "wikicreole.mll"
                                     (
      push_flow c (B.flow (B.pre_elem attribs (List.rev c.pre_content)));
      c.pre_content <- [];
      parse_bol c lexbuf
    )
# 1835 "wikicreole.ml"

  | 2 ->
# 865 "wikicreole.mll"
                                        (
      c.pre_content <- Lexing.lexeme lexbuf :: c.pre_content;
      parse_nowiki c attribs lexbuf
    )
# 1843 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_nowiki_rec c attribs lexbuf __ocaml_lex_state

and parse_extension start name wiki_content args c lexbuf =
   __ocaml_lex_parse_extension_rec start name wiki_content args c lexbuf 162
and __ocaml_lex_parse_extension_rec start name wiki_content args c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 872 "wikicreole.mll"
          (
        if wiki_content
        then ((parse_extension_content_wiki start 0 false "" c lexbuf), args)
        else ((parse_extension_content_nowiki start true "" c lexbuf), args)
      )
# 1859 "wikicreole.ml"

  | 1 ->
# 877 "wikicreole.mll"
                   (
        (None, args)
      )
# 1866 "wikicreole.ml"

  | 2 ->
# 880 "wikicreole.mll"
                                               (
        parse_extension start name wiki_content args c lexbuf
      )
# 1873 "wikicreole.ml"

  | 3 ->
let
# 884 "wikicreole.mll"
                                                        quote
# 1879 "wikicreole.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_curr_pos + -1) in
# 884 "wikicreole.mll"
                                                               (
        let s = Lexing.lexeme lexbuf in
        let i = String.index s '=' in
        let arg_name = String.sub s 0 i in
        let arg_value = parse_arg_value quote "" c lexbuf in
        parse_extension start name wiki_content
          ((arg_name, arg_value)::args) c lexbuf
      )
# 1890 "wikicreole.ml"

  | 4 ->
# 892 "wikicreole.mll"
        (
        ignore
          (if wiki_content
           then ((parse_extension_content_wiki start 0 false "" c lexbuf), args)
           else
             ((parse_extension_content_nowiki start true "" c lexbuf), args));
        (Some ("Syntax error in extension "^name), args)
      )
# 1902 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_extension_rec start name wiki_content args c lexbuf __ocaml_lex_state

and parse_extension_content_wiki start lev nowiki beg c lexbuf =
   __ocaml_lex_parse_extension_content_wiki_rec start lev nowiki beg c lexbuf 176
and __ocaml_lex_parse_extension_content_wiki_rec start lev nowiki beg c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
let
# 903 "wikicreole.mll"
                                  ch
# 1915 "wikicreole.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 1) in
# 903 "wikicreole.mll"
                                      (
          parse_extension_content_wiki
            start lev nowiki (beg^"~"^(String.make 1 ch)) c lexbuf
        )
# 1922 "wikicreole.ml"

  | 1 ->
# 907 "wikicreole.mll"
                                                            (
          let s = Lexing.lexeme lexbuf in
          if nowiki
          then parse_extension_content_wiki start lev nowiki (beg^s) c lexbuf
          else
            let l = String.length s in
            let name = String.sub s 2 (l - 2) in
            let (wiki_content, _) = B.plugin name in
            if wiki_content
            then
              parse_extension_content_wiki
                start (lev+1) false (beg^s) c lexbuf
            else
              let s =
                match
                  parse_extension_content_nowiki start false (beg^s) c lexbuf
                with None -> ">>"
                  | Some s -> s^">>"
              in
              parse_extension_content_wiki start lev false s c lexbuf
        )
# 1947 "wikicreole.ml"

  | 2 ->
# 928 "wikicreole.mll"
              (
          parse_extension_content_wiki
            start lev true (beg^"{{{") c lexbuf
        )
# 1955 "wikicreole.ml"

  | 3 ->
# 932 "wikicreole.mll"
              (
(*VVV Warning: not quotable! *)
          parse_extension_content_wiki start lev false (beg^"}}}") c lexbuf
        )
# 1963 "wikicreole.ml"

  | 4 ->
# 936 "wikicreole.mll"
             (
          if nowiki
          then
            parse_extension_content_wiki start lev nowiki (beg^">>") c lexbuf
          else
            if lev>0
            then
              parse_extension_content_wiki
                start (lev-1) nowiki (beg^">>") c lexbuf
            else Some beg
        )
# 1978 "wikicreole.ml"

  | 5 ->
# 947 "wikicreole.mll"
            (
          Some (beg^" syntax error in wikisyntax") (* or error ?? *)
        )
# 1985 "wikicreole.ml"

  | 6 ->
# 950 "wikicreole.mll"
                                  (
          let s = Lexing.lexeme lexbuf in
          parse_extension_content_wiki start lev nowiki (beg^s) c lexbuf
        )
# 1993 "wikicreole.ml"

  | 7 ->
# 954 "wikicreole.mll"
                                (
          let s = Lexing.lexeme lexbuf in
          parse_extension_content_wiki start lev nowiki (beg^s) c lexbuf
        )
# 2001 "wikicreole.ml"

  | 8 ->
# 958 "wikicreole.mll"
          (
          Ocsigen_messages.warning
            ("Wikicreole: Unrecognized char in extension: "^
               (Lexing.lexeme lexbuf)^".");
          raise Unrecognized_char
        )
# 2011 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_extension_content_wiki_rec start lev nowiki beg c lexbuf __ocaml_lex_state

and parse_extension_content_nowiki start unquote beg c lexbuf =
   __ocaml_lex_parse_extension_content_nowiki_rec start unquote beg c lexbuf 191
and __ocaml_lex_parse_extension_content_nowiki_rec start unquote beg c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 967 "wikicreole.mll"
              (
          let s = if unquote then ">>" else "~>>" in
          parse_extension_content_nowiki start unquote (beg^s) c lexbuf
        )
# 2026 "wikicreole.ml"

  | 1 ->
# 971 "wikicreole.mll"
                     (
          Some beg
        )
# 2033 "wikicreole.ml"

  | 2 ->
# 974 "wikicreole.mll"
                      (
          let s = Lexing.lexeme lexbuf in
          parse_extension_content_nowiki start unquote (beg^s) c lexbuf
        )
# 2041 "wikicreole.ml"

  | 3 ->
# 978 "wikicreole.mll"
                    (
          let s = Lexing.lexeme lexbuf in
          parse_extension_content_nowiki start unquote (beg^s) c lexbuf
        )
# 2049 "wikicreole.ml"

  | 4 ->
# 982 "wikicreole.mll"
          (
          Ocsigen_messages.warning
            ("Wikicreole: Unrecognized char in extension "^
               (Lexing.lexeme lexbuf)^".");
          raise Unrecognized_char
        )
# 2059 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_extension_content_nowiki_rec start unquote beg c lexbuf __ocaml_lex_state

and parse_arg_value quote beg c lexbuf =
   __ocaml_lex_parse_arg_value_rec quote beg c lexbuf 198
and __ocaml_lex_parse_arg_value_rec quote beg c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 992 "wikicreole.mll"
                    (
        raise Eof
      )
# 2073 "wikicreole.ml"

  | 1 ->
let
# 995 "wikicreole.mll"
                               ch
# 2079 "wikicreole.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 1) in
# 995 "wikicreole.mll"
                                   (
        parse_arg_value quote (beg^(String.make 1 ch)) c lexbuf
      )
# 2085 "wikicreole.ml"

  | 2 ->
let
# 998 "wikicreole.mll"
                            ch
# 2091 "wikicreole.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 998 "wikicreole.mll"
                               (
        if ch = quote
        then beg
        else parse_arg_value quote (beg^(String.make 1 ch)) c lexbuf
      )
# 2099 "wikicreole.ml"

  | 3 ->
# 1003 "wikicreole.mll"
                          (
        let s = Lexing.lexeme lexbuf in
        parse_arg_value quote (beg^s) c lexbuf
      )
# 2107 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_arg_value_rec quote beg c lexbuf __ocaml_lex_state

and parse_attribs depth args oldargs c lexbuf =
   __ocaml_lex_parse_attribs_rec depth args oldargs c lexbuf 204
and __ocaml_lex_parse_attribs_rec depth args oldargs c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 1010 "wikicreole.mll"
        (
        args::oldargs
      )
# 2121 "wikicreole.ml"

  | 1 ->
# 1013 "wikicreole.mll"
                                              (
        parse_attribs depth args oldargs c lexbuf
      )
# 2128 "wikicreole.ml"

  | 2 ->
let
# 1017 "wikicreole.mll"
                                                          quote
# 2134 "wikicreole.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_curr_pos + -1) in
# 1017 "wikicreole.mll"
                                                                 (
            let s = Lexing.lexeme lexbuf in
            let i = String.index s '=' in
            let arg_name = String.sub s 0 i in
            let arg_value = parse_arg_value quote "" c lexbuf in
            parse_attribs depth ((arg_name, arg_value)::args) oldargs c lexbuf
          )
# 2144 "wikicreole.ml"

  | 3 ->
# 1024 "wikicreole.mll"
           ( args::oldargs )
# 2149 "wikicreole.ml"

  | 4 ->
# 1025 "wikicreole.mll"
          (
        if depth > 1
        then parse_attribs (depth - 1) [] (args::oldargs) c lexbuf
        else args::oldargs
      )
# 2158 "wikicreole.ml"

  | 5 ->
# 1030 "wikicreole.mll"
            ( []::args::oldargs )
# 2163 "wikicreole.ml"

  | 6 ->
# 1031 "wikicreole.mll"
             ( []::[]::args::oldargs )
# 2168 "wikicreole.ml"

  | 7 ->
# 1032 "wikicreole.mll"
          (
        raise Eof
      )
# 2175 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_parse_attribs_rec depth args oldargs c lexbuf __ocaml_lex_state

;;

# 1037 "wikicreole.mll"
 

let context sectioning param =
  { param;
    italic = false;
    bold = false;
    monospace = false;
    underlined = false;
    linethrough = false;
    subscripted = false;
    superscripted = false;
    heading = false;
    link = false;
    list_level = 0;
    phrasing_mix = [];
    link_content = [];
    pre_content = [];
    list = [];
    descr = [];
    flow = [(0, [])];
    stack = Paragraph [];
    sectioning;
  }

let from_lexbuf_no_preempt sectioning param lexbuf =
  let c = context sectioning param in
  parse_bol c lexbuf;
  match c.flow with
  | [(0, flow)] -> List.rev flow
  | _ -> assert false

end

type ('param, 'res) builder =
    (module Builder with type param = 'param and type flow = 'res)

let from_string (type param) (type flow) ?(sectioning = false) param builder s =
  let module B = (val builder : Builder with type param = param and type flow = flow) in
  let module Wikicreole = Make(B) in
  Wikicreole.from_lexbuf_no_preempt sectioning param (Lexing.from_string s)

open Lwt

let from_channel (type param) (type flow) ?(sectioning = false) param builder ch =
  let module B = (val builder : Builder with type param = param and type flow = flow) in
  let module Wikicreole = Make(B) in
  Lwt_io.read ch >|= fun contents ->
  Wikicreole.from_lexbuf_no_preempt sectioning param (Lexing.from_string contents)


# 2233 "wikicreole.ml"
